==> firefox-56.0b9.source.tar.xz: firefox-56.0b9/layout/inspector/ServoStyleRuleMap.h <==
/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set ts=8 sts=2 et sw=2 tw=80: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef mozilla_ServoStyleRuleMap_h
#define mozilla_ServoStyleRuleMap_h

#include "mozilla/ServoStyleRule.h"

#include "nsDataHashtable.h"
#include "nsICSSLoaderObserver.h"
#include "nsStubDocumentObserver.h"

struct RawServoStyleRule;

namespace mozilla {
class ServoCSSRuleList;
class ServoStyleSet;
namespace css {
class Rule;
} // namespace css

class ServoStyleRuleMap final : public nsStubDocumentObserver
                              , public nsICSSLoaderObserver
{
public:
  NS_DECL_ISUPPORTS

  explicit ServoStyleRuleMap(ServoStyleSet* aStyleSet);

  void EnsureTable();
  ServoStyleRule* Lookup(const RawServoStyleRule* aRawRule) const {
    return mTable.Get(aRawRule);
  }

  // nsIDocumentObserver methods
  void StyleSheetAdded(StyleSheet* aStyleSheet, bool aDocumentSheet) final;
  void StyleSheetRemoved(StyleSheet* aStyleSheet, bool aDocumentSheet) final;
  void StyleSheetApplicableStateChanged(StyleSheet* aStyleSheet) final;
  void StyleRuleAdded(StyleSheet* aStyleSheet, css::Rule* aStyleRule) final;
  void StyleRuleRemoved(StyleSheet* aStyleSheet, css::Rule* aStyleRule) final;

  // nsICSSLoaderObserver
  NS_IMETHOD StyleSheetLoaded(StyleSheet* aSheet,
                              bool aWasAlternate, nsresult aStatus) final;

  size_t SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const;

private:
  ~ServoStyleRuleMap();

  // Since we would never have a document which contains no style rule,
  // we use IsEmpty as an indication whether we need to walk through
  // all stylesheets to fill the table.
  bool IsEmpty() const { return mTable.Count() == 0; }

  void FillTableFromRule(css::Rule* aRule);
  void FillTableFromRuleList(ServoCSSRuleList* aRuleList);
  void FillTableFromStyleSheet(ServoStyleSheet* aSheet);

  typedef nsDataHashtable<nsPtrHashKey<const RawServoStyleRule>,
                          WeakPtr<ServoStyleRule>> Hashtable;
  Hashtable mTable;
  ServoStyleSet* mStyleSet;
};

} // namespace mozilla

#endif // mozilla_ServoStyleRuleMap_h

==> firefox-61.0b5.source.tar.xz: firefox-61.0/layout/inspector/ServoStyleRuleMap.h <==
/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set ts=8 sts=2 et sw=2 tw=80: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef mozilla_ServoStyleRuleMap_h
#define mozilla_ServoStyleRuleMap_h

#include "mozilla/ServoStyleRule.h"
#include "mozilla/StyleSheet.h"

#include "nsDataHashtable.h"

struct RawServoStyleRule;
class nsXBLPrototypeResources;

namespace mozilla {
class ServoCSSRuleList;
class ServoStyleSet;
namespace css {
class Rule;
} // namespace css
namespace dom {
class ShadowRoot;
}
class ServoStyleRuleMap
{
public:
  ServoStyleRuleMap() = default;

  void EnsureTable(ServoStyleSet&);
  void EnsureTable(nsXBLPrototypeResources&);
  void EnsureTable(dom::ShadowRoot&);

  ServoStyleRule* Lookup(const RawServoStyleRule* aRawRule) const
  {
    return mTable.Get(aRawRule);
  }

  void SheetAdded(StyleSheet&);
  void SheetRemoved(StyleSheet&);

  void RuleAdded(StyleSheet& aStyleSheet, css::Rule&);
  void RuleRemoved(StyleSheet& aStyleSheet, css::Rule&);

  size_t SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const;

  ~ServoStyleRuleMap() = default;

private:
  // Since we would never have a document which contains no style rule,
  // we use IsEmpty as an indication whether we need to walk through
  // all stylesheets to fill the table.
  bool IsEmpty() const { return mTable.Count() == 0; }

  void FillTableFromRule(css::Rule&);
  void FillTableFromRuleList(ServoCSSRuleList&);
  void FillTableFromStyleSheet(StyleSheet&);

  typedef nsDataHashtable<nsPtrHashKey<const RawServoStyleRule>,
                          WeakPtr<ServoStyleRule>> Hashtable;
  Hashtable mTable;
};

} // namespace mozilla

#endif // mozilla_ServoStyleRuleMap_h
